require 'active_support/concern'
require 'pureapi/model'

# Provide methods (override, polymorphic) for use data handling
# # => comfortable with gem mongoid

module Pureapi::MongoidModel
  extend ActiveSupport::Concern
  include Pureapi::Model

  # Define constants
  # Need upgrade with nlike operation
  MONGODB_COMPARISON_OPERATORS = {
    ne: '$ne',
    gt: '$gt',
    lt: '$lt',
    gte: '$gte',
    lte: '$lte',
    like: '$regex',
    regex: '$regex',
  }

  MONGODB_LOGICAL_OPERATORS = {
    in: '$in'
  }

  def column_names
    self.attribute_names
  end

  # Rewrite for comfortable with mongoid
  # Comparsion operator conditions method
  # Array +params+ contains hash = {f, o, v}
  # :f is column name
  # :o is comparsion operator
  # :v is value
  def compconds(params = [])
    criterias = self

    queries = {}
    params.each do |param|
      if param[:o] == '='
        queries[param[:f]] = param[:v]
      else
        queries[param[:f]] = queries[param[:f]].is_a?(Hash) ? queries[param[:f]] : {}
        queries[param[:f]][MONGODB_COMPARISON_OPERATORS[Pureapi::Model::COMPARISON_OPERATORS_INVERT[param[:o]]]] = param[:v]
      end
    end

    return criterias.where(queries)
  end

  # Comparsion operator conditions method
  # Array +params+ contains hash = {f, o, v}
  # :f is column name
  # :o is comparsion operator
  # :v is value
  def logicconds(params = [])
    criterias = self

    params.each do |param|
      if param[:o] == Pureapi::Model::LOGICAL_OPERATORS[:in]
        criterias = criterias.in(param[:f] => param[:v])
      end
    end

    return criterias.where({})
  end
end
