require 'active_support/concern'
require 'pureapi/model'

# Provide methods (override, polymorphic) for use data handling
# # => comfortable when use postgresql

module Pureapi::PostgresModel
  extend ActiveSupport::Concern
  include Pureapi::Model

  # New version of +COMPARISON_OPERATORS+ & others
  module Operators
    EQ = { id: :eq, is: '=' }
    NE = { id: :ne, is: '<>' }
    GT = { id: :gt, is: '>' }
    LT = { id: :lt, is: '<' }
    GTE = { id: :gte, is: '>=' }
    LTE = { id: :lte, is: '<=' }
    LIKE = { id: :like, is: 'LIKE' }
    NLIKE = { id: :nlike, is: 'NOT LIKE' }
    REGEXP = { id: :regexp, is: '~' }
    IN = { id: :in, is: 'IN', permit: [] }
    ISNIL = { id: :isnil, is: 'IS NULL' }
    ISNNIL = { id: :isnnil, is: 'IS NOT NULL' }
  end

  REFLECT_OPERATORS = Operators.constants.inject({}) do |memo, const_name|
    item = Operators.const_get(const_name)
    memo[item[:id]] = item[:is]
    memo
  end

  PERMIT_OPERATORS = Operators.constants.inject({}) do |memo, const_name|
    item = Operators.const_get(const_name)
    memo[item[:id]] = item[:permit]
    memo
  end

  # Comparsion operator conditions method
  # Array +params+ contains hash = {f, o, v}
  # :f is column name
  # :o is comparsion operator
  # :v is value
  def compconds(params = [])
    criterias = self

    params.each do |param|
      if param[:o] == COMPARISON_OPERATORS[:eq]
        criterias = criterias.where(param[:f] => param[:v])
      else
        criterias = criterias.where("\"#{self.table_name}\".\"#{param[:f]}\" #{param[:o]} ?", param[:v])
      end
    end

    return criterias.where({})
  end

  # Search by params of attributes of model
  def _self_search only_params
    criterias = self

    only_params.each do |filter|
      case filter[:o]
      when Operators::EQ[:id], Operators::IN[:id]
        criterias = criterias.where(filter[:f] => filter[:v])
      when Operators::ISNIL[:id], Operators::ISNNIL[:id]
        criterias = criterias.where("\"#{self.table_name}\".\"#{filter[:f]}\" #{REFLECT_OPERATORS[filter[:o]]}")
      else
        criterias = criterias.where("\"#{self.table_name}\".\"#{filter[:f]}\" #{REFLECT_OPERATORS[filter[:o]]} ?", filter[:v])
      end
    end

    return criterias.where({})
  end
end
